import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'widgets/map_widget.dart';
import 'widgets/details_widget.dart';
import 'marker_data.dart';
import 'consts.dart';

class DefaultPage extends StatefulWidget {
  const DefaultPage({super.key});

  @override
  _DefaultPageState createState() => _DefaultPageState();
}

class _DefaultPageState extends State<DefaultPage> {
  late Timer _timer;

  List<MarkerTemplate> markerList = [];

  // fetchMarkerTemplate requests data from the update_map endpoint
  Future<void> fetchMarkerTemplate() async {
    try {
      // Custom HTTP client
      HttpClient client = HttpClient()
        ..badCertificateCallback = // NB: temporary disable SSL certificate validation
            (X509Certificate cert, String host, int port) => true;

      // Request makers from API and wait for response
      var request =  await client.getUrl(Uri.parse(serverURI+mapEndpoint));
      var response = await request.close();

      // Attempt to parse json if request is ok
      if (response.statusCode == 200) {
        var responseBody = await response.transform(utf8.decoder).join();
        setState(() {
          // Parse JSON string from response body
          List<dynamic> jsonData = json.decode(responseBody);

          // Convert response from type List<dynamic> to List<MarkerTemplate>
          markerList = jsonData.map((data) => MarkerTemplate.fromJson(data)).toList();
        });
      } else {
        print('Request failed with status: ${response.statusCode}');
      }
    } catch (e) {
      print('Failed to connect to the server: $e');
    }
  }

  // State initializer
  @override
  void initState() {
    super.initState();
    // Call fetchMarkerTemplate when the widget is first created
    fetchMarkerTemplate();

    // Schedule fetchMarkerTemplate to run periodically based on fetchInterval const
    const Duration fiveMinutes = Duration(minutes: fetchInterval);
    _timer = Timer.periodic(fiveMinutes, (timer) {
      fetchMarkerTemplate();
    });
  }

  // Fetch timer
  @override
  void dispose() {
    // Cancel timer on widget termination
    _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('My Page'),
      ),
      body: ListView(
        children: [
          MapContainerWidget(markerList: markerList),
          const SizedBox(height: 30), // Padding between widgets
          DetailsContainerWidget(markerData: markerList.first),
        ],
      ),
    );
  }
}

